import React, {useState, useRef, useEffect} from 'react';
import {
  StyleSheet,
  View,
  Dimensions,
  FlatList,
  Image,
  Text,
  TouchableOpacity,
  ScrollView,
} from 'react-native';
import Svg, {Line} from 'react-native-svg';
import {color_orange, color_orange_2} from '../../assets/colors';
import {onConfigurations} from '../../common/helper';
import SharePreference from '../../common/SharePreference';
import ic_headerphone from '../../assets/images/ic_headphones.png';
import Sound from 'react-native-sound';

const STT = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K'];

export default (props) => {
  let refList1 = useRef(null);
  let refList2 = useRef(null);
  const {
    item,
    dispatch,
    pSound,
    isFinish,
    isResult,
    user_answer: answers,
  } = props;
  let {configurations} = item;
  configurations = JSON.parse(configurations);
  const {correct_answer} = configurations;
  const [totalHeight, setHeight] = useState(0);

  // console.log(27, item);

  const dataCol_1 = correct_answer.col_1.data;
  // dataCol_1.sort();

  const dataCol_2 = correct_answer.col_2.data;
  // dataCol_2.sort();

  const mergeData = [];
  const arrCol1 = [];
  const arrCol2 = [];
  dataCol_1.forEach((col_1, index) => {
    const data = {
      id: index,
      col_1: col_1,
      col_2: '',
      y1: 0,
      y2: 0,
      stt: '',
    };
    mergeData.push(data);
    const newCol1 = {
      value: col_1,
      selected: false,
    };
    arrCol1.push(newCol1);
    const newCol2 = {
      value: dataCol_2[index],
      selected: false,
    };
    arrCol2.push(newCol2);
  });

  arrCol2.sort((a, b) => {
    const x = a.value;
    const y = b.value;
    if (x < y) {
      return -1;
    }
    if (x > y) {
      return 1;
    }
    return 0;
  });
  arrCol2.map((item, index) => {
    item.stt = STT[index];
    return item;
  });

  const [merge, setMergeData] = useState({
    mergeData: mergeData,
    col1: arrCol1,
    col2: arrCol2,
  });

  const [sound, setSound] = useState(null);

  useEffect(() => {
    if (isResult && answers) {
      mergeResult();
    }
  }, []);

  const mergeResult = () => {
    const existed = answers.find((answer) => item.id === answer.question_id);
    // const totalCellLength1 = refList1._listRef._totalCellLength;
    // const totalCellLength2 = refList2._listRef._totalCellLength;

    const totalCellLength1 = 170 * arrCol1.length;
    const heightItem1 = totalCellLength1 / arrCol1.length;

    const totalCellLength2 = 170 * arrCol2.length;
    const heightItem2 = totalCellLength2 / arrCol2.length;

    if (totalCellLength1 > 0 && existed && existed.user_answer.value !== '') {
      setHeight(totalCellLength1);
      const {original_order} = existed.user_answer.value;
      let postion = 0;
      Object.keys(original_order).forEach((key) => {
        if (original_order[key] !== '') {
          const indInCol2 = arrCol2.findIndex((i) => i.value === key);

          mergeData[postion].y1 = (postion + 1) * heightItem1 - heightItem1 / 2;
          mergeData[postion].y2 =
            (indInCol2 + 1) * heightItem2 - heightItem2 / 2;
        }
        postion = postion + 1;
      });
      setMergeData({mergeData: mergeData, col1: arrCol1, col2: arrCol2});
    }
  };

  const itemImage = ({item, index, type, onPress}) => {
    return (
      <View style={styles.container}>
        <Text style={styles.textSTT}>
          {type === 1 ? index + 1 : STT[index]}
        </Text>
        <TouchableOpacity
          disabled={isFinish}
          style={[
            styles.viewImg,
            {backgroundColor: item.selected ? color_orange : '#fff'},
          ]}
          activeOpacity={0.5}
          onPress={() => onPress(item, index)}>
          <Image
            style={styles.image}
            source={{uri: item.value}}
            resizeMode="contain"
          />
        </TouchableOpacity>
      </View>
    );
  };

  const itemText = ({item, index, type, onPress}) => {
    return (
      <View style={styles.container}>
        <Text style={styles.textSTT}>
          {type === 1 ? index + 1 : STT[index]}
        </Text>
        <TouchableOpacity
          disabled={isFinish}
          activeOpacity={0.5}
          onPress={() => {
            onPress(item, index);
          }}
          style={[
            styles.viewText,
            {backgroundColor: item.selected ? color_orange : '#D7E9FF'},
          ]}>
          <Text style={styles.text}>{item.value}</Text>
        </TouchableOpacity>
      </View>
    );
  };

  const itemAudio = ({item, index, type, onPress}) => {
    return (
      <View style={[styles.container]}>
        <Text style={styles.textSTT}>
          {type === 1 ? index + 1 : STT[index]}
        </Text>
        <TouchableOpacity
          disabled={isFinish}
          style={[
            styles.viewAudio,
            {backgroundColor: item.selected ? color_orange : '#D7E9FF'},
          ]}
          activeOpacity={0.5}
          onPress={() => onPress(item, index, 'audio')}>
          <Image
            style={styles.icHeaderphone}
            source={ic_headerphone}
            resizeMode="contain"
          />
        </TouchableOpacity>
      </View>
    );
  };

  const renderItem = (item, index, type) => {
    if (type === 1) {
      const columnType = correct_answer.col_1.type;
      if (columnType === 'image') {
        return itemImage({item, index, type, onPress: onSelectedCol1});
      }
      if (columnType === 'text') {
        return itemText({item, index, type, onPress: onSelectedCol1});
      }
      if (columnType === 'audio') {
        return itemAudio({item, index, type, onPress: onSelectedCol1});
      }
    } else {
      const columnType = correct_answer.col_2.type;
      if (columnType === 'image') {
        return itemImage({item, index, type, onPress: onSelectedCol2});
      }
      if (columnType === 'text') {
        return itemText({item, index, type, onPress: onSelectedCol2});
      }
      if (columnType === 'audio') {
        return itemAudio({item, index, type, onPress: onSelectedCol2});
      }
    }
  };

  const onSelectedCol1 = (item, index, columnType) => {
    const {mergeData, col1, col2} = merge;
    const totalCellLength1 = refList1._listRef._totalCellLength;
    const heightItem1 = totalCellLength1 / col1.length;
    setHeight(totalCellLength1);

    const totalCellLength2 = refList2._listRef._totalCellLength;
    const heightItem2 = totalCellLength2 / col2.length;

    if (columnType && columnType === 'audio') {
      playAudio(item);
    }

    col1.map((i) => {
      if (i.value === item.value) {
        i.selected = i.selected ? false : true;
      } else {
        i.selected = false;
      }
      return i;
    });

    col2.map((i, ind) => {
      if (i.selected) {
        mergeData.map((mer) => {
          if (mer.id === index) {
            mer.col_2 = i.value;
            mer.y1 = (index + 1) * heightItem1 - heightItem1 / 2;
            mer.y2 = (ind + 1) * heightItem2 - heightItem2 / 2;
            mer.stt = i.stt;
          }
          return mer;
        });
        i.selected = false;
        item.selected = false;
        userAnswer(mergeData);
      }
    });
    setMergeData({...merge});
  };

  const onSelectedCol2 = (item, index, columnType) => {
    const {mergeData, col1, col2} = merge;
    const totalCellLength1 = refList1._listRef._totalCellLength;
    const heightItem1 = totalCellLength1 / col1.length;
    setHeight(totalCellLength1);

    const totalCellLength2 = refList2._listRef._totalCellLength;
    const heightItem2 = totalCellLength2 / col2.length;

    if (columnType && columnType === 'audio') {
      playAudio(item);
    }

    col2.map((i) => {
      if (i.value === item.value) {
        i.selected = i.selected ? false : true;
      } else {
        i.selected = false;
      }
      return i;
    });

    col1.map((i, ind) => {
      if (i.selected) {
        mergeData.map((mer) => {
          if (mer.id === ind) {
            mer.col_2 = item.value;
            mer.y1 = (ind + 1) * heightItem1 - heightItem1 / 2;
            mer.y2 = (index + 1) * heightItem2 - heightItem2 / 2;
            mer.stt = item.stt;
          }
          return mer;
        });
        i.selected = false;
        item.selected = false;
        userAnswer(mergeData);
      }
    });
    setMergeData({...merge});
  };

  const userAnswer = (mergeData) => {
    const original_order = {};
    const data1 = [];
    const data2 = [];
    const dataNotSelect = [];
    const correct_order = [];

    arrCol2.forEach((item) => {
      const find = mergeData.find((f) => f.col_2 === item.value);
      if (!find) {
        dataNotSelect.push(item);
      }
    });

    mergeData.forEach((item, index) => {
      const col_2 = item.col_2;
      // console.log(295, col_2);
      data1.push(item.col_1);
      data2.push(arrCol2[index].value);

      if (col_2.length > 0) {
        original_order[col_2] = item.stt;
      } else {
        original_order[dataNotSelect[0].value] = item.stt;
        dataNotSelect.splice(0, 1);
      }
    });

    dataCol_2.forEach((correct) => {
      arrCol2.forEach((origin) => {
        if (correct === origin.value) {
          correct_order.push(origin.stt);
        }
      });
    });

    const col_1 = {
      type: correct_answer.col_1.type,
      data: data1,
    };

    const col_2 = {
      type: correct_answer.col_2.type,
      data: data2,
    };

    const user_answer = {
      value: {
        col_1: col_1,
        col_2: col_2,
        original_order: original_order,
        correct_order: correct_order,
      },
    };
    // console.log(232, user_answer);
    onConfigurations(item, user_answer, dispatch);
  };

  const playAudio = (item) => {
    if (sound) {
      sound.release();
    }
    if (item.selected) {
      return;
    }
    const player = new Sound(item.value, null, (error) => {
      player.play();
      setSound(player);
      pSound(player);
    });
  };

  return (
    <View style={{alignItems: 'center'}}>
      <View style={[styles.configView]}>
        <View style={{width: '35%'}}>
          <FlatList
            ref={(ref) => (refList1 = ref)}
            data={merge.col1}
            renderItem={({item, index}) => renderItem(item, index, 1)}
            keyExtractor={(item, index) => index.toString()}
          />
        </View>
        {merge.mergeData.map((column, index) => {
          if (column.y1 > 0 && column.y2 > 0) {
            return (
              <Svg
                style={{
                  position: 'absolute',
                  left: x1,
                }}
                height={totalHeight}
                width={x2 - x1}
                key={index}>
                <Line
                  x1={0}
                  y1={column.y1}
                  x2={x2 - x1}
                  y2={column.y2}
                  stroke="orange"
                  strokeWidth="2"
                />
              </Svg>
            );
          }
        })}
        <View style={{width: '35%'}}>
          <FlatList
            ref={(ref) => (refList2 = ref)}
            data={merge.col2}
            renderItem={({item, index}) => renderItem(item, index, 2)}
            keyExtractor={(item, index) => index.toString()}
          />
        </View>
      </View>
      {isFinish && (
        <View style={{alignItems: 'center', marginBottom: 50}}>
          <Text style={[styles.textAnswer]}>Đáp án:</Text>
          <ScrollView contentContainerStyle={{flexDirection: 'row'}}>
            {dataCol_2.map((a, index) => {
              let stt = '';
              arrCol2.forEach((b) => {
                if (a === b.value) {
                  stt = b.stt;
                }
              });
              return (
                <Text style={styles.textAnswer} key={index}>
                  {index + 1} - {stt}
                </Text>
              );
            })}
          </ScrollView>
        </View>
      )}
    </View>
  );
};

const width = Dimensions.get('screen').width * 0.55;

const x1 = Dimensions.get('screen').width * 0.2;
const x2 = Dimensions.get('screen').width * 0.35;

const styles = StyleSheet.create({
  configView: {
    flexDirection: 'row',
    width: width,
    alignItems: 'center',
    justifyContent: 'space-between',
    // marginEnd: 50,
  },
  image: {
    flex: 1,
    // marginVertical: 10,
  },
  viewText: {
    flex: 1,
    // borderColor: 'gray',
    // borderWidth: 1,
    borderRadius: 10,
    marginVertical: 10,
    minHeight: 60,
    justifyContent: 'center',
  },
  viewImg: {
    flex: 1,
    // borderColor: 'gray',
    // borderWidth: 1,
    borderRadius: 10,
    marginVertical: 10,
    height: '100%',
    justifyContent: 'center',
  },
  text: {
    fontSize: 16,
    color: 'black',
    alignContent: 'center',
    marginHorizontal: 10,
  },
  textAnswer: {
    fontSize: 18,
    color: 'red',
    alignContent: 'center',
    marginHorizontal: 10,
  },
  textSTT: {
    fontSize: 20,
    color: 'black',
    alignContent: 'center',
    fontWeight: '500',
    marginEnd: 5,
  },
  container: {
    flex: 1,
    minHeight: 150,
    flexDirection: 'row',
    alignItems: 'center',
    marginVertical: 10,
  },
  icHeaderphone: {
    width: 50,
    height: 50,
    alignSelf: 'center',
  },
  viewAudio: {
    flex: 1,
    borderRadius: 10,
    marginVertical: 10,
    height: '90%',
    marginHorizontal: '10%',
    justifyContent: 'center',
  },
});
